@@ -33,6 +33,7 @@ __all__ = [
     # Sony
     "pcsx2",
     "rpcs3",
+    "vita3k",
     # Sega
     "osmose",
     "reicast",
@@ -65,7 +66,7 @@ class NonInstallableRunnerError(LutrisError):
 
 
 def get_runner_module(runner_name):
-    if runner_name not in __all__:
+    if not is_valid_runner_name(runner_name):
         raise InvalidRunnerError("Invalid runner name '%s'" % runner_name)
     module = __import__("lutris.runners.%s" % runner_name, globals(), locals(), [runner_name], 0)
     if not module:
@@ -110,6 +111,10 @@ def get_runner_names():
     return __all__
 
 
+def is_valid_runner_name(runner_name: str) -> bool:
+    return runner_name in __all__
+
+
 def get_runner_human_name(runner_name):
     """Returns a human-readable name for a runner; as a convenience, if the name
     is falsy (None or blank) this returns an empty string. Provides caching for the
