@@ -6,9 +6,9 @@ from gettext import gettext as _
 from pathlib import Path
 
 import requests
-from gi.repository import Gio
 
 from lutris import settings
+from lutris.exceptions import MissingExecutableError
 from lutris.services.base import BaseService
 from lutris.services.service_game import ServiceGame
 from lutris.services.service_media import ServiceMedia
@@ -19,12 +19,12 @@ from lutris.util.strings import slugify
 
 class FlathubBanner(ServiceMedia):
     """Standard size of a Flathub banner"""
+
     service = "flathub"
     size = (128, 128)
     dest_path = os.path.join(settings.CACHE_DIR, "flathub/banners")
-    file_pattern = "%s.png"
-    file_format = "png"
-    url_field = 'iconDesktopUrl'
+    file_patterns = ["%s.png"]
+    url_field = "iconDesktopUrl"
 
     def get_media_url(self, details):
         return details.get(self.url_field)
@@ -32,6 +32,7 @@ class FlathubBanner(ServiceMedia):
 
 class FlathubGame(ServiceGame):
     """Representation of a Flathub game"""
+
     service = "flathub"
 
     @classmethod
@@ -42,10 +43,7 @@ class FlathubGame(ServiceGame):
         service_game.slug = slugify(flathub_game["name"])
         service_game.lutris_slug = slugify(flathub_game["name"])
         service_game.name = flathub_game["name"]
-        service_game.details = {
-            "summary": flathub_game["summary"],
-            "version": flathub_game["currentReleaseVersion"]
-        }
+        service_game.details = {"summary": flathub_game["summary"], "version": flathub_game["currentReleaseVersion"]}
         service_game.runner = "flatpak"
         service_game.details = json.dumps(flathub_game)
         return service_game
@@ -57,9 +55,7 @@ class FlathubService(BaseService):
     id = "flathub"
     name = _("Flathub")
     icon = "flathub"
-    medias = {
-        "banner": FlathubBanner
-    }
+    medias = {"banner": FlathubBanner}
     default_format = "banner"
     api_url = "https://flathub.org/api/v1/apps/category/Game"
     cache_path = os.path.join(settings.CACHE_DIR, "flathub-library.json")
@@ -67,10 +63,7 @@ class FlathubService(BaseService):
     branch = "stable"
     arch = "x86_64"
     install_type = "system"  # can be either system (default) or user
-    install_locations = {
-        "system": "var/lib/flatpak/app/",
-        "user": f"{Path.home()}/.local/share/flatpak/app/"
-    }
+    install_locations = {"system": "var/lib/flatpak/app/", "user": f"{Path.home()}/.local/share/flatpak/app/"}
     runner = "flatpak"
     game_class = FlathubGame
 
@@ -88,7 +81,7 @@ class FlathubService(BaseService):
         flatpak_spawn_abspath = shutil.which("flatpak-spawn")
         if flatpak_spawn_abspath:
             return [flatpak_spawn_abspath, "--host", "flatpak"]
-        raise RuntimeError("No flatpak or flatpak-spawn found")
+        raise MissingExecutableError(_("No flatpak or flatpak-spawn found"))
 
     def load(self):
         """Load the available games from Flathub"""
@@ -107,21 +100,24 @@ class FlathubService(BaseService):
         logger.debug("Installing %s from service %s", app_id, self.id)
         # Check if Flathub repo is active on the system
         if not self.is_flathub_remote_active():
-            logger.error("Flathub is not configured on the system. Visit https://flatpak.org/setup/ for instructions.")
-            return
+            raise RuntimeError(
+                _("Flathub is not configured on the system. Visit https://flatpak.org/setup/ for instructions.")
+            )
         # Install the game
-        service_installers = self.get_installers_from_api(app_id)
-        if not service_installers:
-            service_installers = [self.generate_installer(db_game)]
-        application = Gio.Application.get_default()
-        application.show_installer_window(service_installers, service=self, appid=app_id)
+        self.install_from_api(db_game, app_id)
 
     def get_installed_apps(self):
         """Get list of installed Flathub apps"""
         try:
             flatpak_cmd = self.get_flatpak_cmd()
-            process = subprocess.run(flatpak_cmd + ["list", "--app", "--columns=application"],
-                                     capture_output=True, check=True, encoding="utf-8", text=True, timeout=5.0)
+            process = subprocess.run(
+                flatpak_cmd + ["list", "--app", "--columns=application"],
+                capture_output=True,
+                check=True,
+                encoding="utf-8",
+                text=True,
+                timeout=5.0,
+            )
             return process.stdout.splitlines() or []
         except (TimeoutError, subprocess.CalledProcessError) as err:
             logger.exception("Error occurred while getting installed flatpak apps: %s", err)
@@ -134,7 +130,7 @@ class FlathubService(BaseService):
             logger.warning("Remotes not found, Flathub considered installed")
             return True
         for remote in remotes:
-            if 'flathub' in remote.values():
+            if "flathub" in remote.values():
                 return True
         return False
 
@@ -142,16 +138,18 @@ class FlathubService(BaseService):
         """Get a list of dictionaries containing name, title and url"""
         try:
             flatpak_cmd = self.get_flatpak_cmd()
-            process = subprocess.run(flatpak_cmd + ["remotes", "--columns=name,title,url"],
-                                     capture_output=True, check=True, encoding="utf-8", text=True, timeout=5.0)
+            process = subprocess.run(
+                flatpak_cmd + ["remotes", "--columns=name,title,url"],
+                capture_output=True,
+                check=True,
+                encoding="utf-8",
+                text=True,
+                timeout=5.0,
+            )
             entries = []
             for line in process.stdout.splitlines():
                 cols = line.split("\t")
-                entries.append({
-                    "name": cols[0].lower(),
-                    "title": cols[1].lower(),
-                    "url": cols[2]
-                })
+                entries.append({"name": cols[0].lower(), "title": cols[1].lower(), "url": cols[2]})
             return entries
         except (TimeoutError, subprocess.CalledProcessError) as err:
             logger.exception("Error occurred while getting installed flatpak apps: %s", err)
@@ -162,39 +160,41 @@ class FlathubService(BaseService):
         flatpak_cmd = self.get_flatpak_cmd()
         return {
             "appid": db_game["appid"],
-            "game_slug": slugify(db_game["name"]),
+            "game_slug": self.get_installed_slug(db_game),
             "slug": slugify(db_game["name"]) + "-" + self.id,
             "name": db_game["name"],
             "version": "Flathub",
-            "runner": self.runner,
+            "runner": self.get_installed_runner_name(db_game),
             "script": {
                 "game": {
                     "appid": db_game["appid"],
                     "arch": self.arch,
                     "branch": self.branch,
-                    "install_type": self.install_type
-                },
-                "system": {
-                    "disable_runtime": True
+                    "install_type": self.install_type,
                 },
+                "system": {"disable_runtime": True},
                 "require-binaries": flatpak_cmd[0],
                 "installer": [
                     {
-                        "execute":
-                        {
+                        "execute": {
                             "file": flatpak_cmd[0],
-                            "args": " ".join(flatpak_cmd[1:]) + f" install --app --noninteractive flathub "
-                                    f"app/{db_game['appid']}/{self.arch}/{self.branch}",
-                            "disable_runtime": True
+                            "args": " ".join(flatpak_cmd[1:])
+                            + f" install --{self.install_type} --app --noninteractive flathub "
+                            f"app/{db_game['appid']}/{self.arch}/{self.branch}",
+                            "disable_runtime": True,
                         }
                     }
-                ]
-            }
+                ],
+            },
         }
 
+    def get_installed_runner_name(self, db_game):
+        return self.runner
+
     def get_game_directory(self, _installer):
-        install_type, application, arch, branch = (_installer["script"]["game"][key] for key in
-                                                   ("install_type", "application", "arch", "branch"))
+        install_type, application, arch, branch = (
+            _installer["script"]["game"][key] for key in ("install_type", "application", "arch", "branch")
+        )
         return os.path.join(self.install_locations[install_type], application, arch, branch)
 
     # def add_installed_games(self):
