@@ -1,12 +1,13 @@
 """Options list for system config."""
+
 import os
 from collections import OrderedDict
 from gettext import gettext as _
 
 from lutris import runners
 from lutris.util import linux, system
-from lutris.util.display import DISPLAY_MANAGER, SCREEN_SAVER_INHIBITOR, USE_DRI_PRIME, is_compositing_enabled
-from lutris.util.system import get_vk_icd_file_sets, get_vulkan_gpu_name
+from lutris.util.display import DISPLAY_MANAGER, SCREEN_SAVER_INHIBITOR, is_compositing_enabled
+from lutris.util.graphics.gpu import GPUS
 
 
 def get_resolution_choices():
@@ -45,6 +46,13 @@ def get_locale_choices():
     ]
 
 
+def get_gpu_list():
+    choices = [(_("Auto"), "")]
+    for card, gpu in GPUS.items():
+        choices.append((gpu.short_name, card))
+    return choices
+
+
 def get_output_choices():
     """Return list of outputs for drop-downs"""
     displays = DISPLAY_MANAGER.get_display_names()
@@ -67,56 +75,6 @@ def get_output_list():
     return choices
 
 
-def get_optirun_choices():
-    """Return menu choices (label, value) for Optimus"""
-    choices = [(_("Off"), "off")]
-    if system.can_find_executable("primusrun"):
-        choices.append(("primusrun", "primusrun"))
-    if system.can_find_executable("optirun"):
-        choices.append(("optirun/virtualgl", "optirun"))
-    if system.can_find_executable("pvkrun"):
-        choices.append(("primus vk", "pvkrun"))
-    return choices
-
-
-def get_vk_icd_choices():
-    """Return available Vulkan ICD loaders"""
-    # fallback in case any ICDs don't match a known type
-    icd_file_sets = get_vk_icd_file_sets()
-
-    intel_files = ":".join(icd_file_sets["intel"])
-    amdradv_files = ":".join(icd_file_sets["amdradv"])
-    nvidia_files = ":".join(icd_file_sets["nvidia"])
-    amdvlk_files = ":".join(icd_file_sets["amdvlk"])
-    amdvlkpro_files = ":".join(icd_file_sets["amdvlkpro"])
-    unknown_files = ":".join(icd_file_sets["unknown"])
-
-    # default choice should always be blank so the env var gets left as is
-    # This ensures Lutris doesn't change the vulkan loader behavior unless you select
-    # a specific ICD from the list, to avoid surprises
-    choices = [("Unspecified", "")]
-
-    if intel_files:
-        choices.append(("Intel Open Source (MESA: ANV)", intel_files))
-    if amdradv_files:
-        choices.append(("AMD RADV Open Source (MESA: RADV)", amdradv_files))
-    if nvidia_files:
-        choices.append(("Nvidia Proprietary", nvidia_files))
-    if amdvlk_files:
-        if not amdvlkpro_files:
-            choices.append(("AMDVLK/AMDGPU-PRO Proprietary", amdvlk_files))
-        else:
-            choices.append(("AMDVLK Open source", amdvlk_files))
-    if amdvlkpro_files:
-        choices.append(("AMDGPU-PRO Proprietary", amdvlkpro_files))
-    if unknown_files:
-        choices.append(("Unknown Vendor", unknown_files))
-
-    choices = [(prefix + ": " + get_vulkan_gpu_name(files, USE_DRI_PRIME), files) for prefix, files in choices]
-
-    return choices
-
-
 system_options = [  # pylint: disable=invalid-name
     {
         "section": _("Lutris"),
@@ -126,7 +84,7 @@ system_options = [  # pylint: disable=invalid-name
         "warn_if_non_writable_parent": True,
         "default": os.path.expanduser("~/Games"),
         "scope": ["runner", "system"],
-        "help": _("The default folder where you install your games.")
+        "help": _("The default folder where you install your games."),
     },
     {
         "section": _("Lutris"),
@@ -134,9 +92,11 @@ system_options = [  # pylint: disable=invalid-name
         "type": "bool",
         "label": _("Disable Lutris Runtime"),
         "default": False,
-        "help": _("The Lutris Runtime loads some libraries before running the "
-                  "game, which can cause some incompatibilities in some cases. "
-                  "Check this option to disable it."),
+        "help": _(
+            "The Lutris Runtime loads some libraries before running the "
+            "game, which can cause some incompatibilities in some cases. "
+            "Check this option to disable it."
+        ),
     },
     {
         "section": _("Lutris"),
@@ -144,10 +104,103 @@ system_options = [  # pylint: disable=invalid-name
         "type": "bool",
         "label": _("Prefer system libraries"),
         "default": True,
-        "help": _("When the runtime is enabled, prioritize the system libraries"
-                  " over the provided ones."),
+        "help": _("When the runtime is enabled, prioritize the system libraries" " over the provided ones."),
+    },
+    {
+        "section": _("Display"),
+        "option": "gpu",
+        "type": "choice",
+        "label": _("GPU"),
+        "choices": get_gpu_list,
+        "default": "",
+        "condition": lambda: len(GPUS) > 1,
+        "help": _("GPU to use to run games"),
+    },
+    {
+        "section": _("Display"),
+        "option": "mangohud",
+        "type": "bool",
+        "label": _("FPS counter (MangoHud)"),
+        "default": False,
+        "condition": system.can_find_executable("mangohud"),
+        "help": _("Display the game's FPS + other information. Requires MangoHud to be installed."),
+    },
+    {
+        "section": _("Display"),
+        "option": "reset_desktop",
+        "type": "bool",
+        "label": _("Restore resolution on game exit"),
+        "default": False,
+        "advanced": True,
+        "help": _(
+            "Some games don't restore your screen resolution when \n"
+            "closed or when they crash. This is when this option comes \n"
+            "into play to save your bacon."
+        ),
+    },
+    {
+        "section": _("Display"),
+        "option": "disable_compositor",
+        "label": _("Disable desktop effects"),
+        "type": "bool",
+        "default": False,
+        "advanced": True,
+        "condition": is_compositing_enabled(),
+        "help": _("Disable desktop effects while game is running, " "reducing stuttering and increasing performance"),
+    },
+    {
+        "section": _("Display"),
+        "option": "disable_screen_saver",
+        "label": _("Disable screen saver"),
+        "type": "bool",
+        "default": SCREEN_SAVER_INHIBITOR is not None,
+        "advanced": True,
+        "condition": SCREEN_SAVER_INHIBITOR is not None,
+        "help": _(
+            "Disable the screen saver while a game is running. "
+            "Requires the screen saver's functionality "
+            "to be exposed over DBus."
+        ),
+    },
+    {
+        "section": _("Display"),
+        "option": "sdl_video_fullscreen",
+        "type": "choice",
+        "label": _("SDL 1.2 Fullscreen Monitor"),
+        "choices": get_output_list,
+        "default": "off",
+        "advanced": True,
+        "help": _(
+            "Hint SDL 1.2 games to use a specific monitor when going "
+            "fullscreen by setting the SDL_VIDEO_FULLSCREEN "
+            "environment variable"
+        ),
+    },
+    {
+        "section": _("Display"),
+        "option": "display",
+        "type": "choice",
+        "label": _("Turn off monitors except"),
+        "choices": get_output_choices,
+        "default": "off",
+        "advanced": True,
+        "help": _(
+            "Only keep the selected screen active while the game is "
+            "running. \n"
+            "This is useful if you have a dual-screen setup, and are \n"
+            "having display issues when running a game in fullscreen."
+        ),
+    },
+    {
+        "section": _("Display"),
+        "option": "resolution",
+        "type": "choice",
+        "label": _("Switch resolution to"),
+        "advanced": True,
+        "choices": get_resolution_choices,
+        "default": "off",
+        "help": _("Switch to this screen resolution while the game is running."),
     },
-
     {
         "section": _("Gamescope"),
         "option": "gamescope",
@@ -155,8 +208,17 @@ system_options = [  # pylint: disable=invalid-name
         "label": _("Enable Gamescope"),
         "default": False,
         "condition": system.can_find_executable("gamescope") and linux.LINUX_SYSTEM.nvidia_gamescope_support(),
-        "help": _("Use gamescope to draw the game window isolated from your desktop.\n"
-                  "Toggle fullscreen: Super + F"),
+        "help": _("Use gamescope to draw the game window isolated from your desktop.\n" "Toggle fullscreen: Super + F"),
+    },
+    {
+        "section": _("Gamescope"),
+        "option": "gamescope_hdr",
+        "type": "bool",
+        "label": _("Enable HDR (Experimental)"),
+        "advanced": False,
+        "default": False,
+        "condition": bool(system.can_find_executable("gamescope")),
+        "help": _("Enable HDR for games that support it.\bn" "Requires Plasma 6 and VK_hdr_layer."),
     },
     {
         "section": _("Gamescope"),
@@ -166,9 +228,11 @@ system_options = [  # pylint: disable=invalid-name
         "advanced": True,
         "default": False,
         "condition": bool(system.can_find_executable("gamescope")),
-        "help": _("Always use relative mouse mode instead of flipping\n"
-                  "dependent on cursor visibility\n"
-                  "Can help with games where the player's camera faces the floor"),
+        "help": _(
+            "Always use relative mouse mode instead of flipping\n"
+            "dependent on cursor visibility\n"
+            "Can help with games where the player's camera faces the floor"
+        ),
     },
     {
         "section": _("Gamescope"),
@@ -178,10 +242,12 @@ system_options = [  # pylint: disable=invalid-name
         "choices": DISPLAY_MANAGER.get_resolutions,
         "advanced": True,
         "condition": system.can_find_executable("gamescope"),
-        "help": _("Set the resolution used by gamescope.\n"
-                  "Resizing the gamescope window will update these settings.\n"
-                  "\n"
-                  "<b>Custom Resolutions:</b> (width)x(height)"),
+        "help": _(
+            "Set the resolution used by gamescope.\n"
+            "Resizing the gamescope window will update these settings.\n"
+            "\n"
+            "<b>Custom Resolutions:</b> (width)x(height)"
+        ),
     },
     {
         "section": _("Gamescope"),
@@ -190,9 +256,7 @@ system_options = [  # pylint: disable=invalid-name
         "label": _("Game Resolution"),
         "choices": DISPLAY_MANAGER.get_resolutions,
         "condition": system.can_find_executable("gamescope"),
-        "help": _("Set the maximum resolution used by the game.\n"
-                  "\n"
-                  "<b>Custom Resolutions:</b> (width)x(height)"),
+        "help": _("Set the maximum resolution used by the game.\n" "\n" "<b>Custom Resolutions:</b> (width)x(height)"),
     },
     {
         "section": _("Gamescope"),
@@ -206,8 +270,7 @@ system_options = [  # pylint: disable=invalid-name
         ),
         "default": "-f",
         "condition": system.can_find_executable("gamescope"),
-        "help": _("Run gamescope in fullscreen, windowed or borderless mode\n"
-                  "Toggle fullscreen : Super + F"),
+        "help": _("Run gamescope in fullscreen, windowed or borderless mode\n" "Toggle fullscreen : Super + F"),
     },
     {
         "section": _("Gamescope"),
@@ -216,14 +279,15 @@ system_options = [  # pylint: disable=invalid-name
         "advanced": True,
         "type": "string",
         "condition": system.can_find_executable("gamescope"),
-        "help": _("Use AMD FidelityFX™ Super Resolution 1.0 for upscaling.\n"
-                  "Upscaler sharpness from 0 (max) to 20 (min)."),
+        "help": _(
+            "Use AMD FidelityFX™ Super Resolution 1.0 for upscaling.\n" "Upscaler sharpness from 0 (max) to 20 (min)."
+        ),
     },
     {
         "section": _("Gamescope"),
         "option": "gamescope_fps_limiter",
-        "label": _("FPS Limiter"),
-        "advanced": True,
+        "label": _("Framerate Limiter"),
+        "advanced": False,
         "type": "string",
         "condition": system.can_find_executable("gamescope"),
         "help": _("Set a frame-rate limit for gamescope specified in frames per second."),
@@ -235,8 +299,9 @@ system_options = [  # pylint: disable=invalid-name
         "advanced": True,
         "type": "string",
         "condition": system.can_find_executable("gamescope"),
-        "help": _("Set additional flags for gamescope (if available).\n"
-                  "See 'gamescope --help' for a full list of options."),
+        "help": _(
+            "Set additional flags for gamescope (if available).\n" "See 'gamescope --help' for a full list of options."
+        ),
     },
     {
         "section": _("CPU"),
@@ -263,112 +328,6 @@ system_options = [  # pylint: disable=invalid-name
         "label": _("Enable Feral GameMode"),
         "help": _("Request a set of optimisations be temporarily applied to the host OS"),
     },
-    {
-        "section": _("Display"),
-        "option": "mangohud",
-        "type": "bool",
-        "label": _("FPS counter (MangoHud)"),
-        "default": False,
-        "condition": system.can_find_executable("mangohud"),
-        "help": _("Display the game's FPS + other information. Requires MangoHud to be installed."),
-    },
-    {
-        "section": _("Display"),
-        "option": "reset_desktop",
-        "type": "bool",
-        "label": _("Restore resolution on game exit"),
-        "default": False,
-        "help": _("Some games don't restore your screen resolution when \n"
-                  "closed or when they crash. This is when this option comes \n"
-                  "into play to save your bacon."),
-    },
-    {
-        "section": _("Display"),
-        "option": "restore_gamma",
-        "type": "bool",
-        "default": False,
-        "label": _("Restore gamma on game exit"),
-        "advanced": True,
-        "help": _("Some games don't correctly restores gamma on exit, making "
-                  "your display too bright. Select this option to correct it."),
-    },
-    {
-        "section": _("Display"),
-        "option": "disable_compositor",
-        "label": _("Disable desktop effects"),
-        "type": "bool",
-        "default": False,
-        "advanced": True,
-        "condition": is_compositing_enabled(),
-        "help": _("Disable desktop effects while game is running, "
-                  "reducing stuttering and increasing performance"),
-    },
-    {
-        "section": _("Display"),
-        "option": "disable_screen_saver",
-        "label": _("Disable screen saver"),
-        "type": "bool",
-        "default": SCREEN_SAVER_INHIBITOR is not None,
-        "advanced": False,
-        "condition": SCREEN_SAVER_INHIBITOR is not None,
-        "help": _("Disable the screen saver while a game is running. "
-                  "Requires the screen saver's functionality "
-                  "to be exposed over DBus."),
-    },
-
-    {
-        "section": _("Display"),
-        "option": "fps_limit",
-        "type": "string",
-        "size": "small",
-        "label": _("FPS limit"),
-        "condition": system.can_find_executable("strangle"),
-        "help": _("Limit the game's FPS using libstrangle"),
-    },
-    {
-        "section": _("Display"),
-        "option": "sdl_video_fullscreen",
-        "type": "choice",
-        "label": _("SDL 1.2 Fullscreen Monitor"),
-        "choices": get_output_list,
-        "default": "off",
-        "advanced": True,
-        "help": _("Hint SDL 1.2 games to use a specific monitor when going "
-                  "fullscreen by setting the SDL_VIDEO_FULLSCREEN "
-                  "environment variable"),
-    },
-    {
-        "section": _("Display"),
-        "option": "display",
-        "type": "choice",
-        "label": _("Turn off monitors except"),
-        "choices": get_output_choices,
-        "default": "off",
-        "advanced": True,
-        "help": _("Only keep the selected screen active while the game is "
-                  "running. \n"
-                  "This is useful if you have a dual-screen setup, and are \n"
-                  "having display issues when running a game in fullscreen."),
-    },
-    {
-        "section": _("Display"),
-        "option": "resolution",
-        "type": "choice",
-        "label": _("Switch resolution to"),
-        "choices": get_resolution_choices,
-        "default": "off",
-        "help": _("Switch to this screen resolution while the game is running."),
-    },
-    {
-        "section": _("Audio"),
-        "option": "reset_pulse",
-        "type": "bool",
-        "label": _("Reset PulseAudio"),
-        "default": False,
-        "advanced": True,
-        "condition": system.can_find_executable("pulseaudio"),
-        "help": _("Restart PulseAudio before launching the game."),
-    },
     {
         "section": _("Audio"),
         "option": "pulse_latency",
@@ -377,8 +336,7 @@ system_options = [  # pylint: disable=invalid-name
         "default": False,
         "advanced": True,
         "condition": system.can_find_executable("pulseaudio") or system.can_find_executable("pipewire-pulse"),
-        "help": _("Set the environment variable PULSE_LATENCY_MSEC=60 "
-                  "to improve audio quality on some games"),
+        "help": _("Set the environment variable PULSE_LATENCY_MSEC=60 " "to improve audio quality on some games"),
     },
     {
         "section": _("Input"),
@@ -397,71 +355,16 @@ system_options = [  # pylint: disable=invalid-name
         "advanced": True,
         "help": _("Path to an AntiMicroX profile file"),
     },
-
     {
         "section": _("Input"),
         "option": "sdl_gamecontrollerconfig",
         "type": "string",
         "label": _("SDL2 gamepad mapping"),
         "advanced": True,
-        "help": _("SDL_GAMECONTROLLERCONFIG mapping string or path to a custom "
-                  "gamecontrollerdb.txt file containing mappings."),
-    },
-    {
-        "section": _("Multi-GPU"),
-        "option": "prime",
-        "type": "bool",
-        "default": False,
-        "condition": True,
-        "label": _("Enable NVIDIA Prime Render Offload"),
-        "help": _("If you have the latest NVIDIA driver and the properly patched xorg-server (see "
-                  "https://download.nvidia.com/XFree86/Linux-x86_64/435.17/README/primerenderoffload.html"
-                  "), you can launch a game on your NVIDIA GPU by toggling this switch. This will apply "
-                  "__NV_PRIME_RENDER_OFFLOAD=1 and "
-                  "__GLX_VENDOR_LIBRARY_NAME=nvidia environment variables.")
-    },
-    {
-        "section": _("Multi-GPU"),
-        "option": "dri_prime",
-        "type": "bool",
-        "default": USE_DRI_PRIME,
-        "condition": USE_DRI_PRIME,
-        "label": _("Use discrete graphics"),
-        "advanced": True,
-        "help": _("If you have open source graphic drivers (Mesa), selecting this "
-                  "option will run the game with the 'DRI_PRIME=1' environment variable, "
-                  "activating your discrete graphic chip for high 3D "
-                  "performance."),
-    },
-    {
-        "section": _("Multi-GPU"),
-        "option": "optimus",
-        "type": "choice",
-        "default": "off",
-        "choices": get_optirun_choices,
-        "label": _("Optimus launcher (NVIDIA Optimus laptops)"),
-        "advanced": True,
-        "help": _("If you have installed the primus or bumblebee packages, "
-                  "select what launcher will run the game with the command, "
-                  "activating your NVIDIA graphic chip for high 3D "
-                  "performance. primusrun normally has better performance, but"
-                  "optirun/virtualgl works better for more games."
-                  "Primus VK provide vulkan support under bumblebee."),
-    },
-    {
-        "section": _("Multi-GPU"),
-        "option": "vk_icd",
-        "type": "choice",
-        # Default is "" which does not set the VK_ICD_FILENAMES env var
-        # (Matches "Unspecified" in dropdown)
-        "default": "",
-        "choices": get_vk_icd_choices,
-        "label": _("Vulkan ICD loader"),
-        "advanced": True,
-        "help": _("The ICD loader is a library that is placed between a Vulkan "
-                  "application and any number of Vulkan drivers, in order to support "
-                  "multiple drivers and the instance-level functionality that works "
-                  "across these drivers.")
+        "help": _(
+            "SDL_GAMECONTROLLERCONFIG mapping string or path to a custom "
+            "gamecontrollerdb.txt file containing mappings."
+        ),
     },
     {
         "section": _("Text based games"),
@@ -470,8 +373,10 @@ system_options = [  # pylint: disable=invalid-name
         "type": "bool",
         "default": False,
         "advanced": True,
-        "help": _("Enable a terminal for text-based games. "
-                  "Only useful for ASCII based games. May cause issues with graphical games."),
+        "help": _(
+            "Enable a terminal for text-based games. "
+            "Only useful for ASCII based games. May cause issues with graphical games."
+        ),
     },
     {
         "section": _("Text based games"),
@@ -481,9 +386,11 @@ system_options = [  # pylint: disable=invalid-name
         "choices": linux.get_terminal_apps,
         "default": linux.get_default_terminal(),
         "advanced": True,
-        "help": _("The terminal emulator used with the CLI mode. "
-                  "Choose from the list of detected terminal apps or enter "
-                  "the terminal's command or path."),
+        "help": _(
+            "The terminal emulator used with the CLI mode. "
+            "Choose from the list of detected terminal apps or enter "
+            "the terminal's command or path."
+        ),
     },
     {
         "section": _("Game execution"),
@@ -497,9 +404,7 @@ system_options = [  # pylint: disable=invalid-name
         "option": "locale",
         "type": "choice_with_entry",
         "label": _("Locale"),
-        "choices": (
-            get_locale_choices
-        ),
+        "choices": (get_locale_choices),
         "default": "",
         "advanced": False,
         "help": _("Can be used to force certain locale for an app. Fixes encoding issues in legacy software."),
@@ -510,8 +415,7 @@ system_options = [  # pylint: disable=invalid-name
         "type": "string",
         "label": _("Command prefix"),
         "advanced": True,
-        "help": _("Command line instructions to add in front of the game's "
-                  "execution command."),
+        "help": _("Command line instructions to add in front of the game's " "execution command."),
     },
     {
         "section": _("Game execution"),
@@ -552,10 +456,12 @@ system_options = [  # pylint: disable=invalid-name
         "type": "string",
         "label": _("Include processes"),
         "advanced": True,
-        "help": _("What processes to include in process monitoring. "
-                  "This is to override the built-in exclude list.\n"
-                  "Space-separated list, processes including spaces "
-                  "can be wrapped in quotation marks."),
+        "help": _(
+            "What processes to include in process monitoring. "
+            "This is to override the built-in exclude list.\n"
+            "Space-separated list, processes including spaces "
+            "can be wrapped in quotation marks."
+        ),
     },
     {
         "section": _("Game execution"),
@@ -563,11 +469,13 @@ system_options = [  # pylint: disable=invalid-name
         "type": "string",
         "label": _("Exclude processes"),
         "advanced": True,
-        "help": _("What processes to exclude in process monitoring. "
-                  "For example background processes that stick around "
-                  "after the game has been closed.\n"
-                  "Space-separated list, processes including spaces "
-                  "can be wrapped in quotation marks."),
+        "help": _(
+            "What processes to exclude in process monitoring. "
+            "For example background processes that stick around "
+            "after the game has been closed.\n"
+            "Space-separated list, processes including spaces "
+            "can be wrapped in quotation marks."
+        ),
     },
     {
         "section": _("Game execution"),
@@ -575,11 +483,12 @@ system_options = [  # pylint: disable=invalid-name
         "type": "string",
         "label": _("Killswitch file"),
         "advanced": True,
-        "help": _("Path to a file which will stop the game when deleted \n"
-                  "(usually /dev/input/js0 to stop the game on joystick "
-                  "unplugging)"),
+        "help": _(
+            "Path to a file which will stop the game when deleted \n"
+            "(usually /dev/input/js0 to stop the game on joystick "
+            "unplugging)"
+        ),
     },
-
     {
         "section": _("Xephyr (Deprecated, use Gamescope)"),
         "option": "xephyr",
