@@ -1,4 +1,5 @@
 """Process management"""
+
 import os
 import shlex
 import sys
@@ -24,6 +25,8 @@ SYSTEM_PROCESSES = {
     "mscorsvw.exe",
     "iexplore.exe",
     "winedbg.exe",
+    "tabtip.exe",
+    "conhost.exe",
 }
 
 
@@ -60,7 +63,7 @@ class ProcessWatcher:
 
     def iterate_processes(self):
         for child in self.iterate_children():
-            if child.state == 'Z':
+            if child.state == "Z":
                 continue
 
             if child.name and child.name not in self.unmonitored_processes:
