@@ -1,4 +1,5 @@
 """XDG shortcuts handling"""
+
 import os
 import shlex
 import shutil
@@ -15,7 +16,7 @@ from lutris.util.log import logger
 
 
 def get_lutris_executable():
-    if LINUX_SYSTEM.is_flatpak:
+    if LINUX_SYSTEM.is_flatpak():
         return "flatpak run net.lutris.Lutris"
     return "lutris"
 
@@ -58,11 +59,11 @@ def create_launcher(game_slug, game_id, game_name, launch_config_name=None, desk
     desktop_dir = GLib.get_user_special_dir(GLib.UserDirectory.DIRECTORY_DESKTOP)
     lutris_executable = get_lutris_executable()
 
-    url = format_installer_url({
-        "action": "rungameid",
-        "game_slug": game_id,
-        "launch_config_name": launch_config_name
-    })
+    url = format_installer_url({"action": "rungameid", "game_slug": game_id, "launch_config_name": launch_config_name})
+
+    # Quote URL for the shell but *also* quote %, which indicates a desktop file
+    # field code in the Exec key.
+    command = f"{lutris_executable} {shlex.quote(url)}".replace("%", "%%")
 
     launcher_content = dedent(
         """
@@ -70,29 +71,19 @@ def create_launcher(game_slug, game_id, game_name, launch_config_name=None, desk
         Type=Application
         Name={}
         Icon={}
-        Exec=env LUTRIS_SKIP_INIT=1 {} {}
+        Exec=env LUTRIS_SKIP_INIT=1 {}
         Categories=Game
-        """.format(
-            game_name,
-            "lutris_{}".format(game_slug),
-            lutris_executable,
-            shlex.quote(url)
-        )
+        """.format(game_name, f"lutris_{game_slug}", command)
     )
 
     launcher_filename = get_xdg_basename(game_slug, game_id)
     tmp_launcher_path = os.path.join(CACHE_DIR, launcher_filename)
-    with open(tmp_launcher_path, "w", encoding='utf-8') as tmp_launcher:
+    with open(tmp_launcher_path, "w", encoding="utf-8") as tmp_launcher:
         tmp_launcher.write(launcher_content)
         tmp_launcher.close()
     os.chmod(
         tmp_launcher_path,
-        stat.S_IREAD
-        | stat.S_IWRITE
-        | stat.S_IEXEC
-        | stat.S_IRGRP
-        | stat.S_IWGRP
-        | stat.S_IXGRP,
+        stat.S_IREAD | stat.S_IWRITE | stat.S_IEXEC | stat.S_IRGRP | stat.S_IWGRP | stat.S_IXGRP,
     )
 
     if desktop:
@@ -101,7 +92,7 @@ def create_launcher(game_slug, game_id, game_name, launch_config_name=None, desk
         logger.debug("Creating Desktop icon in %s", launcher_path)
         shutil.copy(tmp_launcher_path, launcher_path)
     if menu:
-        user_dir = os.path.expanduser("~/.local/share") if LINUX_SYSTEM.is_flatpak else GLib.get_user_data_dir()
+        user_dir = os.path.expanduser("~/.local/share") if LINUX_SYSTEM.is_flatpak() else GLib.get_user_data_dir()
         menu_path = os.path.join(user_dir, "applications")
         os.makedirs(menu_path, exist_ok=True)
         launcher_path = os.path.join(menu_path, launcher_filename)
